gleam/io

Values

pub fn print(string: String) -> Nil

Writes a string to standard output (stdout).

If you want your output to be printed on its own line see println.

Example

assert io.print("Hi mum") == Nil
// Hi mum
pub fn print_error(string: String) -> Nil

Writes a string to standard error (stderr).

If you want your output to be printed on its own line see println_error.

Example

assert io.print_error("Hi pop") == Nil
// Hi pop
pub fn println(string: String) -> Nil

Writes a string to standard output (stdout), appending a newline to the end.

Example

assert io.println("Hi mum") == Nil
// Hi mum
pub fn println_error(string: String) -> Nil

Writes a string to standard error (stderr), appending a newline to the end.

Example

assert io.println_error("Hi pop") == Nil
// Hi pop
Search Document